from enum import Enum
import dateutil.parser

from nexosisapi.column_metadata import ColumnMetadata
from nexosisapi.status import Status


class ImportType(Enum):
    s3 = 0


class ImportResponse(object):
    def __init__(self, data_dict=None):
        self._import_id = data_dict['importId']
        self._type = ImportType[data_dict['type']]
        self._status = Status[data_dict['status']]
        self._dataset_name = data_dict['dataSetName']
        self._requested_date = dateutil.parser.parse(data_dict['requestedDate'])
        self._status_history = data_dict['statusHistory']
        self._links = data_dict['links']
        self._parameters = data_dict['parameters']
        self._messages = data_dict['messages']
        self._column_metadata = {key: ColumnMetadata(value) for (key, value) in data_dict.get('metadata', {}).items()}

    @property
    def import_id(self):
        return self._import_id

    @property
    def type(self):
        return self._type

    @property
    def status(self):
        return self._status

    @property
    def dataset_name(self):
        return self._dataset_name

    @property
    def status_history(self):
        return self._status_history

    @property
    def links(self):
        return self._links

    @property
    def parameters(self):
        return self._parameters

    @property
    def messages(self):
        return self._messages

    @property
    def column_metadata(self):
        return self._column_metadata

    @property
    def requested_date(self):
        return self._requested_date



from datetime import datetime, date
import json
import requests

from enum import Enum
from nexosisapi.column_metadata import ColumnMetadata
from .client_error import ClientError


def _process_response(response):
    if len(response.content) == 0:
        return None, 0, None

    # content type is probably something like 'application/json; charset: utf-8', so this processes that
    content_type_value = response.headers['content-type']
    content_type = content_type_value.split(';')[0]

    if content_type == 'application/json':
        return response.json(), response.status_code, response.headers
    else:
        return response.content, response.status_code, response.headers


def _json_encode(obj):
    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    if isinstance(obj, ColumnMetadata):
        json = {'dataType': obj.data_type, 'role': obj.role}
        if(obj.imputation is not None):
            json['imputation'] = obj.imputation
        if(obj.aggregation is not None):
            json['aggregation'] = obj.aggregation
        return json
    if isinstance(obj, Enum):
        return obj.name
    raise TypeError("Type %s not serializable" % type(obj))


class HttpClient(object):
    def __init__(self, key, uri):
        self._key = key
        self._uri = uri

    def _generate_headers(self):
        return {
            'api-key': self._key,
            'User-Agent': 'Nexosis-Python-API-Client/1.0',
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

    def _get_uri(self, fragment):
        if fragment.startswith('/'):
            fragment = fragment[1:]
        return '%s/%s' % (self._uri, fragment)

    def _process_args(self, args):
        # if headers specified, then generate defaults and update with the ones specified
        # or just set them if none specified
        default_headers = self._generate_headers()
        if 'headers' in args:
            user_headers = args['headers']
            args['headers'] = default_headers
            args['headers'].update(user_headers)
        else:
            args['headers'] = default_headers

        # copy data to json for proper serialization
        if 'data' in args and args['headers']['Content-Type'] == 'application/json':
            args['data'] = json.dumps(args['data'], default=_json_encode)
       
        return args

    def request_with_headers(self, verb, uri_path, **kwargs):
        response = requests.request(verb, self._get_uri(uri_path), **self._process_args(kwargs))
        if response.ok:
            return _process_response(response)
        else:
            error = response.json()
            raise ClientError(uri_path, response.status_code, error)

    # TODO: should be a better way to do this re: the 'verb' argument
    def request(self, verb, uri_path, **kwargs):
        response, _, _ = self.request_with_headers(verb, uri_path, **kwargs)
        return response



from enum import Enum


class ColumnType(Enum):
    string = 0
    numeric = 1
    logical = 2
    date = 3
    numericMeasure = -1


class Role(Enum):
    none = 0
    timestamp = 1
    target = 2
    feature = 3


class Imputation(Enum):
    zeroes = 0
    mean = 1
    median = 2
    mode = 3


class Aggregation(Enum):
    sum = 0
    mean = 1
    median = 2
    mode = 3


class ColumnMetadata(object):
    """The data describing a column in a dataset."""

    def __init__(self, data_dict={}):
        """Create an instance with the data or defaults

        Defaults to data_type = ColumnType.string and role = ColumnRole.none

        :arg dict data_dict: the dictionary containing the data for this object
        """

        impute = data_dict.get('imputation')
        aggregate = data_dict.get('aggregation')

        self._data_type = ColumnType[data_dict.get('dataType') or 'string']
        self._role = Role[data_dict.get('role') or 'none']
        self._imputation = Imputation[impute] if impute is not None else None
        self._aggregation = Aggregation[aggregate] if aggregate is not None else None
        
        

    @property
    def data_type(self):
        """Gets the data_type of the column.

        :return: The data_type of the column.
        :rtype: ColumnType
        """
        return self._data_type

    @property
    def role(self):
        """Gets the role of the column.

        :return: The role of this column.
        :rtype: ColumnRole
        """
        return self._role

    @property
    def imputation(self):
        """Gets the imputation strategy of the column.

        :return: The imputation strategy of the column.
        :rtype: Imputation
        """
        return self._imputation

    @property
    def aggregation(self):
        """Gets the aggregation strategy of the column.

        :return: The aggregation strategy of the column.
        :rtype: Aggregation
        """
        return self._aggregation




import csv
import time
import os
import dateutil.parser
import unittest

from nexosisapi import Client
from nexosisapi.column_metadata import ColumnMetadata
from nexosisapi.import_response import ImportType
from nexosisapi.status import Status


class ImportIntegrationTests(unittest.TestCase):
    def setUp(self):
        self.test_client = Client(key=os.environ["NEXOSIS_API_TESTKEY"], uri=os.environ["NEXOSIS_API_TESTURI"])

        metadata = {'sales': ColumnMetadata({'dataType': 'numeric', 'role': 'target'}),
                    'transactions': ColumnMetadata({'dataType': 'numeric', 'role': 'none'}),
                    'timeStamp': ColumnMetadata({'dataType': 'date', 'role': 'timestamp'})}

        self.import_response = self.test_client.imports.import_from_s3('test-python-import', 'nexosis-sample-data',
                                                                       'LocationA.csv', 'us-east-1', metadata)

    def tearDown(self):
        self.test_client.sessions.remove(self.import_response.import_id)

    def test_import_from_s3(self):
        self.assertIsNotNone(self.import_response)
        self.assertEqual(Status.requested, self.import_response.status)

    def test_get_import(self):
        import_ = self.test_client.imports.get(self.import_response.import_id)

        self.assertEqual(self.import_response.import_id, import_.import_id)

    def test_list_import(self):
        import_list = self.test_client.imports.list()

        self.assertGreater(len(import_list), 0)



from nexosisapi.column_metadata import ColumnMetadata


class Dataset(object):
    def __init__(self, data_dict=None):
        """
        A Dataset is the representation of your data as stored by the Nexosis API

        :arg dict data_dict: the dictionary containing the data for this object
        """
        if data_dict is None:
            data_dict = {}
        self._data = data_dict.get('data')
        self._metadata = {key: ColumnMetadata(value) for (key, value) in data_dict.get('metadata', {}).items()}
        self._links = data_dict.get('links')

    @property
    def data(self):
        """Gets the data for this Dataset.

        :return: The data.
        :rtype: list(dict)
        """
        return self._data

    @property
    def metadata(self):
        """Gets the column metadata for this Dataset.

        :return: The data.
        :rtype: list(dict)
        """
        return self._data

    @property
    def links(self):
        """Gets the links for this Dataset.

        :return: The links.
        :rtype: list(dict)
        """
        return self._links



class FakeHttpClient(object):
    def __init__(self, return_value):
        self._return = return_value
        self._verb = None
        self._uri = None
        self._args = None

    @property
    def verb(self):
        return self._verb

    @property
    def uri(self):
        return self._uri

    @property
    def args(self):
        return self._args

    def request(self, verb, uri, **kwargs):
        self._verb = verb
        self._uri = uri
        self._args = kwargs
        return self._return



import os
import unittest

import nexosisapi


class BasicTests(unittest.TestCase):
    def setUp(self):
        self.test_client = nexosisapi.Client(key=os.environ["NEXOSIS_API_TESTKEY"], uri=os.environ["NEXOSIS_API_TESTURI"])

    def test_can_create_client_with_env_key(self):
        target = nexosisapi.Client()
        self.assertIsNotNone(target._key)

    def test_can_override_client_base_uri(self):
        target = nexosisapi.Client(uri='https://something.example.com/v1')
        self.assertEqual(target._uri, 'https://something.example.com/v1', 'uri not set by constructor')

    def test_api_returns_current_balance(self):
        actual = self.test_client.get_account_balance()
        self.assertIsNotNone(actual)
        self.assertTrue(actual > 0)



import os
import tempfile
import unittest
from datetime import datetime
import csv
import json

from nexosisapi import Client, ClientError
from nexosisapi.column_metadata import ColumnMetadata, ColumnType, Role, Imputation, Aggregation


class DatasetsIntegrationTests(unittest.TestCase):
    def setUp(self):
        self.test_client = Client(key=os.environ["NEXOSIS_API_TESTKEY"],
                                  uri=os.environ["NEXOSIS_API_TESTURI"])
        self.ds_name = "data-%s" % datetime.now().strftime('%Y-%m-%d-%H-%M-%S')

        with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/data.csv')) as f:
            csv_data = csv.DictReader(f)
            self.data = [dict(d) for d in csv_data]

    def tearDown(self):
        try:
            self.test_client.datasets.remove(self.ds_name)
        except ClientError:
            pass

    def test_create(self):
        result = self.test_client.datasets.create(self.ds_name, self.data)

        self.assertEqual(self.ds_name, result.name)
        self.assertEqual(ColumnType.numeric, result.column_metadata['observed'].data_type)
        self.assertEqual(Role.target, result.column_metadata['observed'].role)
        self.assertEqual(ColumnType.date, result.column_metadata['timestamp'].data_type)
        self.assertEqual(Role.timestamp, result.column_metadata['timestamp'].role)

    def test_create_with_measure_type(self):
        metadata = {'observed': ColumnMetadata({'dataType': 'numericMeasure', 'role': 'target'}),
                    'timestamp': ColumnMetadata({'dataType': 'date', 'role': 'timestamp'})}
        result = self.test_client.datasets.create(self.ds_name, self.data, metadata)

        self.assertEqual(Imputation.mean, result.column_metadata['observed'].imputation)
        self.assertEqual(Aggregation.mean, result.column_metadata['observed'].aggregation)

    def test_create_with_metadata(self):
        metadata = {'observed': ColumnMetadata({'dataType': 'string', 'role': 'none'}),
                    'timestamp': ColumnMetadata({'dataType': 'date', 'role': 'timestamp'})}
        result = self.test_client.datasets.create(self.ds_name, self.data, metadata)

        self.assertEqual(self.ds_name, result.name)
        self.assertEqual(metadata['observed'].data_type, result.column_metadata['observed'].data_type)
        self.assertEqual(metadata['observed'].role, result.column_metadata['observed'].role)
        self.assertEqual(metadata['timestamp'].data_type, result.column_metadata['timestamp'].data_type)
        self.assertEqual(metadata['timestamp'].role, result.column_metadata['timestamp'].role)

    def test_create_assign_imputation_aggregation(self):
        metadata = {'observed': ColumnMetadata({'dataType': 'numeric', 'role': 'target', 'imputation' : 'mode', 'aggregation' : 'median'}),
                    'timestamp': ColumnMetadata({'dataType': 'date', 'role': 'timestamp'})}

        result = self.test_client.datasets.create(self.ds_name, self.data, metadata)

        self.assertEqual(self.ds_name, result.name)
        self.assertEqual(Aggregation.median, result.column_metadata['observed'].aggregation)
        self.assertEqual(Imputation.mode, result.column_metadata['observed'].imputation)


    def test_create_adding_data_adds_more_data(self):
        # initial data added, items 0-9
        self.test_client.datasets.create(self.ds_name, self.data[0:10])
        result_first = self.test_client.datasets.get(self.ds_name)
        self.assertEqual(10, len(result_first.data))

        # more data added, items 10-end
        self.test_client.datasets.create(self.ds_name, self.data[10:])

        result = self.test_client.datasets.get(self.ds_name)

        # make sure the saved data has extended the initial data upload to the full data set
        self.assertEqual(len(self.data), len(result.data))

    def test_create_from_csv(self):
        with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/data.csv')) as f:
            result = self.test_client.datasets.create_csv(self.ds_name, f)

        self.assertEqual(self.ds_name, result.name)

        check = self.test_client.datasets.get(self.ds_name)
        self.assertEqual(len(self.data), len(check.data))

    def test_list_datasets(self):
        self.test_client.datasets.create(self.ds_name, self.data)

        ds_list = self.test_client.datasets.list()

        self.assertIn(self.ds_name, map(lambda x: x.name, ds_list))

    def test_list_with_filtering(self):
        self.test_client.datasets.create(self.ds_name, self.data)

        ds_list = self.test_client.datasets.list(self.ds_name[:10])

        self.assertIn(self.ds_name, [x.name for x in ds_list])

    def test_get(self):
        self.test_client.datasets.create(self.ds_name, self.data)

        dataset = self.test_client.datasets.get(self.ds_name)

        def sorter(d): return d['timestamp']

        self.assertListEqual(sorted(self.data, key=sorter), sorted(dataset.data, key=sorter))

    def test_get_filtered(self):
        self.test_client.datasets.create(self.ds_name, self.data)

        dataset = self.test_client.datasets.get(self.ds_name,
                                                start_date=datetime.strptime('2008-06-01', '%Y-%m-%d'),
                                                end_date=datetime.strptime('2008-06-30', '%Y-%m-%d'))

        self.assertEqual(30, len(dataset.data))

    def test_get_as_csv(self):
        self.test_client.datasets.create(self.ds_name, self.data)

        temp_file = os.path.join(tempfile.gettempdir(),
                                 '%s%s.tmp' % (tempfile.gettempprefix(), datetime.now().strftime('%f')))

        with open(temp_file, 'wb') as f:
            self.test_client.datasets.get_csv(self.ds_name, f)

        with open(temp_file, 'r') as f:
            # the +1 is for the headers written to the file
            self.assertEqual(len(self.data) + 1, len(f.readlines()))

        os.remove(temp_file)

    def test_remove(self):
        self.test_client.datasets.create(self.ds_name, self.data)

        self.test_client.datasets.remove(self.ds_name)

        try:
            self.test_client.datasets.get(self.ds_name)
        except ClientError as e:
            self.assertEqual(404, e.status)
        else:
            self.assertFalse(True, 'expected ClientError for data set that does not exist')

    def test_remove_some_data(self):
        self.test_client.datasets.create(self.ds_name, self.data)

        self.test_client.datasets.remove(self.ds_name, end_date=datetime.strptime('2008-06-30', '%Y-%m-%d'))

        partial = self.test_client.datasets.get(self.ds_name)
        self.assertEqual(len(self.data) - 30, len(partial.data))



class ErrorResponse(object):
    def __init__(self, body):
        self._status = body.get('statusCode')
        self._message = body.get('message')
        self._error_type = body.get('errorType')
        self._details = body.get('errorDetails')

    @property
    def status(self):
        return self._status

    @property
    def error_type(self):
        return self._error_type

    @property
    def message(self):
        return self._message

    @property
    def details(self):
        return self._details


class ClientError(Exception):
    def __init__(self, url, status, body):
        self._url = url
        self._status = status
        self._error_details = ErrorResponse(body)

    @property
    def url(self):
        return self._url

    @property
    def status(self):
        return self._status

    @property
    def error_details(self):
        return self._error_details



from nexosisapi.import_response import ImportResponse


class Imports(object):
    def __init__(self, client):
        self._client = client

    def list(self):
        response = self._client.request('GET', '/imports')
        return [ImportResponse(r) for r in response.get('items', [])]

    def import_from_s3(self, dataset_name, bucket_name, path, region='us-east-1', metadata=None):
        if dataset_name is None:
            raise ValueError('dataset_name is required and was not provided')
        if bucket_name is None:
            raise ValueError('bucket_name is required and was not provided')
        if path is None:
            raise ValueError('path is required and was not provided')

        body = {
            'dataSetName': dataset_name,
            'bucket': bucket_name,
            'path': path,
            'region': region,
            'columns': metadata
        }
        response = self._client.request('POST', '/imports/s3', data=body)

        return ImportResponse(response)

    def get(self, import_id):
        if import_id is None:
            raise ValueError('import_id is required and was not provided')

        response = self._client.request('GET', '/imports/%s' % import_id)
        return ImportResponse(response)



from nexosisapi.session import SessionResult, SessionResponse
from nexosisapi.time_interval import TimeInterval


class Sessions(object):
    """Session based API operations"""
    def __init__(self, client):
        self._client = client

    def _create_session(self, dataset_name, action_type, target_column, event_name,
                        start_date, end_date, result_interval, is_estimate=False, column_metadata=None,
                        callback_url=None):
        if dataset_name is None:
            raise ValueError('dataset_name is required and was not provided')
        if target_column is None:
            raise ValueError('target_column is required and was not provided')
        if start_date is None:
            raise ValueError('start_date is required and was not provided')
        if end_date is None:
            raise ValueError('end_date is required and was not provided')

        return self._client.request_with_headers('POST', 'sessions/%s' % action_type,
                                                 params={
                                                     'dataSetName': dataset_name,
                                                     'targetColumn': target_column,
                                                     'eventName': event_name,
                                                     'startDate': start_date,
                                                     'endDate': end_date,
                                                     'isEstimate': is_estimate,
                                                     'resultInterval': result_interval.name,
                                                     'callbackUrl': callback_url
                                                 },
                                                 data={
                                                     'dataSetName': dataset_name,
                                                     'columns': column_metadata
                                                 })

    def create_forecast(self, dataset_name, target_column, start_date, end_date, result_interval=TimeInterval.day,
                        callback_url=None):
        """Create a new forecast for a dataset

        :param str dataset_name: the name of the dataset to forecast on
        :param str target_column: the column from the dataset to forecast over
        :param datetime start_date: the first datetime of the forecast
        :param datetime end_date: the last datetime of the forecast
        :param TimeInterval result_interval: the interval between predictions in the results
        :param str callback_url: the url to callback to on session status change events

        :return the session description
        :rtype: SessionResponse
        """
        response, _, headers = self._create_session(dataset_name, 'forecast', target_column, None, start_date, end_date,
                                                    result_interval, callback_url=callback_url)
        return SessionResponse(response, headers)

    def analyze_impact(self, dataset_name, target_column, event_name, start_date, end_date,
                       result_interval=TimeInterval.day, callback_url=None):
        """Create a new impact analysis on a dataset

        :param str dataset_name: the name of the dataset to forecast on
        :param str target_column: the column from the dataset to forecast over
        :param str event_name: the name of this analysis
        :param datetime start_date: the first datetime of the forecast
        :param datetime end_date: the last datetime of the forecast
        :param TimeInterval result_interval: the interval between predictions in the results
        :param str callback_url: the url to callback to on session status change events

        :return the session description
        :rtype: SessionResponse
        """
        response, _, headers = self._create_session(dataset_name, 'impact', target_column, event_name, start_date,
                                                    end_date, result_interval, callback_url=callback_url)
        return SessionResponse(response, headers)

    def estimate_forecast(self, dataset_name, target_column, start_date, end_date, result_interval=TimeInterval.day):
        """Estimate a new forecast for a dataset

        :param str dataset_name: the name of the dataset to forecast on
        :param str target_column: the column from the dataset to forecast over
        :param datetime start_date: the first datetime of the forecast
        :param datetime end_date: the last datetime of the forecast
        :param TimeInterval result_interval: the interval between predictions in the results

        :return the session description
        :rtype: SessionResponse
        """
        response, _, headers = self._create_session(dataset_name, 'forecast', target_column, None, start_date, end_date,
                                                    result_interval, is_estimate=True)
        return SessionResponse(response, headers)

    def estimate_impact(self, dataset_name, target_column, event_name, start_date, end_date,
                        result_interval=TimeInterval.day):
        """Estimate an impact analysis on a dataset

        :param str dataset_name: the name of the dataset to forecast on
        :param str target_column: the column from the dataset to forecast over
        :param str event_name: the name of this analysis
        :param datetime start_date: the first datetime of the forecast
        :param datetime end_date: the last datetime of the forecast
        :param TimeInterval result_interval: the interval between predictions in the results

        :return the session description
        :rtype: SessionResponse
        """
        response, _, headers = self._create_session(dataset_name, 'impact', target_column, event_name, start_date,
                                                    end_date, result_interval, is_estimate=True)
        return SessionResponse(response, headers)

    def list(self, dataset_name=None, event_name=None, requested_after=None, requested_before=None, session_type=None):
        """list the created sessions, optionally filtering on session parameters

        :param str dataset_name: filter on the name of the dataset used
        :param str event_name: filter on the event name given when running an impact analysis
        :param datetime requested_before: only include sessions created before this date
        :param datetime requested_after: only include sessions created after this date
        :param SessionType session_type: filter on the type of session

        :returns a list of :class:`SessionResponses`
        :rtype list
        """
        query = {
            'dataSetName': dataset_name,
            'eventName': event_name,
            'requestedBefore': requested_before,
            'requestedAfter': requested_after,
            'sessionType': session_type
        }
        response, _, headers = self._client.request_with_headers('GET', 'sessions', params=query)

        return [SessionResponse(item, headers) for item in response.get('items', [])]

    def remove(self, session_id):
        """Remove a session based on the session id

        :param str session_id: the session to remove
        """
        if session_id is None:
            raise ValueError('session_id is required and was not provided')

        self._client.request('DELETE', 'sessions/%s' % session_id)

    def remove_sessions(self, **kwargs):
        self._client.request('DELETE', 'sessions', params=kwargs)

    def get_results(self, session_id):
        """Get the results of a session based on the session id

        :param str session_id: the session to get results for

        :returns the results of computation run by the session
        :rtype SessionResult
        """
        if session_id is None:
            raise ValueError('session_id is required and was not provided')

        response = self._client.request('GET', 'sessions/%s/results' % session_id)
        return SessionResult(response)

    def get(self, session_id):
        """Get a session based on the session id

        :param str session_id: the session to get

        :returns the information about the session
        :rtype SessionResponse
        """
        if session_id is None:
            raise ValueError('session_id is required and was not provided')

        response, _, headers = self._client.request_with_headers('GET', 'sessions/%s' % session_id)
        return SessionResponse(response, headers)



from nexosisapi.dataset import Dataset
from nexosisapi.dataset_summary import DatasetSummary


class Datasets(object):
    """Dataset based API operations"""
    def __init__(self, base_client):
        self._client = base_client

    def create(self, dataset_name, data, metadata=None):
        """Save data in a named dataset

        :param str dataset_name: the name of the dataset
        :param list data: a :class:`list` of :class:`dict` where each dict is the set of values in the data set.
        :param dict metadata: a dict of `str` keys to :class:`ColumnMetadata` items where the string key matches one
            of the keys from the entries in the data dicts

        :return: a :class:`DatasetSummary` describing the dataset
        :rtype: DatasetSummary
        """
        return self._create(dataset_name, {'data': data, 'columns': metadata}, 'application/json')

    def create_csv(self, dataset_name, csv_file):
        """Save data from a CSV file in a named dataset

        :param str dataset_name: the name of the dataset
        :param file csv_file: an open file to read the csv data from

        :return: a :class:`DatasetSummary` describing the dataset
        :rtype: DatasetSummary
        """
        return self._create(dataset_name, csv_file.read(), 'text/csv')

    def _create(self, dataset_name, content, content_type):
        if dataset_name is None:
            raise ValueError('dataset_name is required and was not provided')

        response = self._client.request('PUT', '/data/%s' % dataset_name, data=content,
                                        headers={'Content-Type': content_type})

        return DatasetSummary(response)

    def list(self, partial_name=''):
        """Get the list of saved datasets, optionally filtering by name

        :param str partial_name:
        :return: a :class:`list` of DatasetSummary objects representing the dataset stored
        :rtype: list
        """
        listing = self._client.request('GET', '/data', params={'partialName': partial_name})
        return [DatasetSummary(item) for item in listing.get('items', [])]

    def get(self, dataset_name, page_number=0, page_size=1000, start_date=None, end_date=None, include=None):
        """Get the data stored in a data set

        :param str dataset_name: name of the dataset
        :param int page_number: zero-based page number of results to retrieve
        :param int page_size: count of results to retrieve in each page (default 1000, max 1000).
        :param datetime start_date: the first date to return in the response
        :param datetime end_date: the last date to return in the response
        :param include: string or array of strings specifying the names of the columns from the dataset to return
        :return: a :class:`Dataset` with the data queried
        :rtype: Dataset
        """
        if dataset_name is None:
            raise ValueError('dataset_name is required and was not provided')

        params = Datasets.process_parameters(page_number, page_size, start_date, end_date, include)

        dataset = self._client.request('GET', '/data/%s' % dataset_name, params=params)

        return Dataset(dataset)

    def get_csv(self, dataset_name, csv_file, page_number=0, page_size=1000, start_date=None, end_date=None,
                include=None):
        """Get the data stored in a data set, and write it to a file

        :param str dataset_name: name of the dataset
        :param FileIO csv_file: an open, writeable text file to save the data to
        :param int page_number: zero-based page number of results to retrieve
        :param int page_size: count of results to retrieve in each page (default 1000, max 1000).
        :param datetime start_date: the first date to return in the response
        :param datetime end_date: the last date to return in the response
        :param include: string or array of strings specifying the names of the columns from the dataset to return
        """
        if dataset_name is None:
            raise ValueError('dataset_name is required and was not provided')
        if csv_file is None:
            raise ValueError('csv_file is required and was not provided')

        params = Datasets.process_parameters(page_number, page_size, start_date, end_date, include)

        data = self._client.request('GET', '/data/%s' % dataset_name, params=params, headers={'Accept': 'text/csv'})

        csv_file.write(data)

    def remove(self, dataset_name, start_date=None, end_date=None, cascade=None):
        """Delete a dataset by name

        :param str dataset_name: name of the dataset
        :param datetime start_date: the starting date to remove from the dataset
        :param datetime end_date: the ending date to remove from the dataset
        :param list cascade: set the cascade options of the removal.

        The cascade list can contain 'forecast', 'session' or both.
        When 'forecast' is included, all related forecasts will be removed.
        When 'session' is include, all related sessions will be removed.
        """
        if dataset_name is None:
            raise ValueError('dataset_name is required and was not provided')

        filter_options = {}
        if start_date:
            filter_options['startDate'] = start_date
        if end_date:
            filter_options['endDate'] = end_date
        if cascade:
            filter_options['cascade'] = cascade

        self._client.request('DELETE', '/data/%s' % dataset_name, params=filter_options)

    @staticmethod
    def process_parameters(page_number, page_size, start_date, end_date, include):
        params = {'page': page_number, 'pageSize': page_size}
        if start_date is not None:
            params['startDate'] = start_date
        if end_date is not None:
            params['endDate'] = end_date
        if include is not None:
            params['include'] = include
        return params



from enum import Enum

import dateutil.parser

from nexosisapi.column_metadata import ColumnMetadata
from nexosisapi.status import Status
from nexosisapi.time_interval import TimeInterval


class SessionType(Enum):
    import_ = 0,
    forecast = 1,
    impact = 2


class Session(object):
    def __init__(self, data_dict=None):
        if data_dict is None:
            data_dict = {}

        self._session_id = data_dict['sessionId']
        self._type = SessionType[data_dict['type']]
        self._status = Status[data_dict['status']]
        self._status_history = data_dict['statusHistory']
        self._dataset_name = data_dict['dataSetName']
        self._target_column = data_dict['targetColumn']
        self._start_date = dateutil.parser.parse(data_dict['startDate'])
        self._end_date = dateutil.parser.parse(data_dict['endDate'])
        self._requested_date = dateutil.parser.parse(data_dict['requestedDate'])
        self._links = data_dict['links']
        self._is_estimate = bool(data_dict['isEstimate'])
        self._extra_parameters = data_dict['extraParameters']
        self._result_interval = TimeInterval[data_dict['resultInterval']] \
            if 'resultInterval' in data_dict.keys() and data_dict['resultInterval'] \
            else TimeInterval.day
        self._column_metadata = {key: ColumnMetadata(value) for (key, value) in data_dict.get('metadata', {}).items()}

    @property
    def session_id(self):
        return self._session_id

    @property
    def type(self):
        return self._type

    @property
    def status(self):
        return self._status

    @property
    def status_history(self):
        return self._status_history

    @property
    def dataset_name(self):
        return self._dataset_name

    @property
    def target_column(self):
        return self._target_column

    @property
    def start_date(self):
        return self._start_date

    @property
    def end_date(self):
        return self._end_date

    @property
    def requested_date(self):
        return self._requested_date

    @property
    def links(self):
        return self._links

    @property
    def is_estimate(self):
        return self._is_estimate

    @property
    def result_interval(self):
        return self._result_interval

    @property
    def column_metadata(self):
        return self._column_metadata

    @property
    def extra_parameters(self):
        return self._extra_parameters


class SessionResponse(Session):
    def __init__(self, data_dict, headers):
        super(SessionResponse, self).__init__(data_dict)
        self._cost = headers.get('nexosis-request-cost')
        self._balance = headers.get('nexosis-account-balance')

    @property
    def cost(self):
        return self._cost

    @property
    def balance(self):
        return self._balance


class SessionResult(Session):
    def __init__(self, data_dict):
        super(SessionResult, self).__init__(data_dict)

        self._metrics = data_dict['metrics']
        self._data = data_dict['data']

    @property
    def metrics(self):
        return self._metrics

    @property
    def data(self):
        return self._data



import os
from setuptools import find_packages, setup

with open('README.md') as f:
    long_description = f.read()

build_number = os.getenv('TRAVIS_BUILD_NUMBER', '1')

setup(name='nexosisapi',
      version='1.0.' + build_number,
      description='Python Client for the Nexosis API',
      long_description='This software is provided as a way to include Nexosis API functionality in your own Python '
                       'software. You can read about the Nexosis API at https://developers.nexosis.com',
      url='http://github.com/nexosis/nexosisclient-py',
      author='Nexosis',
      author_email='support@nexosis.com',
      license='Apache 2.0',
      packages=find_packages(),
      install_requires=[
          'requests', 'enum34', 'python-dateutil'
      ],
      test_suite='nexosisapi.tests.all',
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      include_package_data=True,
      zip_safe=False)



import json
import os

import requests

from .sessions import Sessions
from .imports import Imports
from .datasets import Datasets
from .http_client import HttpClient

from nexosisapi import NEXOSIS_API_KEY


class Client(object):
    def __init__(self, key=None, uri='https://ml.nexosis.com/v1', client=None):
        self._key = key or os.environ[NEXOSIS_API_KEY]
        if uri.endswith('/'):
            uri = uri[:-1]
        self._uri = uri

        if client is None:
            client = HttpClient(key, uri)
        self._client = client
        self._datasets = Datasets(self._client)
        self._imports = Imports(self._client)
        self._sessions = Sessions(self._client)

    @property
    def datasets(self):
        return self._datasets

    @property
    def imports(self):
        return self._imports

    @property
    def sessions(self):
        return self._sessions

    def get_account_balance(self):
        """get_account_balance"""
        response, status, headers = self._client.request_with_headers('GET', 'data')
        if status == 200:
            header = headers['nexosis-account-balance']
            if header is not None:
                value = header.split(' ')[0]
                return float(value)

        response.raise_for_status()




from nexosisapi.column_metadata import ColumnMetadata


class DatasetSummary(object):
    def __init__(self, data_dict):
        self._name = data_dict['dataSetName']
        self._column_metadata = {key: ColumnMetadata(value) for key, value in data_dict.get('columns', {}).items()}

    @property
    def name(self):
        return self._name

    @property
    def column_metadata(self):
        return self._column_metadata



import csv
import time
import os

import dateutil.parser
import unittest

from nexosisapi import Client, ClientError
from nexosisapi.session import SessionType
from nexosisapi.status import Status


class SessionIntegrationTests(unittest.TestCase):
    def setUp(self):
        self.forecast = None
        self.impact = None
        self.test_client = Client(key=os.environ["NEXOSIS_API_TESTKEY"], uri=os.environ["NEXOSIS_API_TESTURI"])
        self.ds_name = "data-sessions-integration"
        with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/data.csv')) as f:
            csv_data = csv.DictReader(f)
            self.data = [dict(d) for d in csv_data]

        try:
            self.test_client.datasets.get(self.ds_name)
        except ClientError as e:
            if e.status == 404:
                self.test_client.datasets.create(self.ds_name, self.data)
            else:
                raise

        self._setup_sessions()

    def tearDown(self):
        try:
            session_list = self.test_client.sessions.list()

            for session in session_list:
                if session.dataset_name != self.ds_name:
                    self.test_client.sessions.remove(session.session_id)

            dataset_list = self.test_client.datasets.list('test-session-integration')

            for dataset in dataset_list:
                self.test_client.datasets.remove(dataset.name)

        except ClientError:
            pass

    def test_list_sessions(self):
        results = self.test_client.sessions.list(self.ds_name)

        name = set([s.dataset_name for s in results])

        self.assertEqual(1, len(name))

    def test_create_forecast(self):
        self.test_client.datasets.create('test-session-integration-forecast', self.data)
        forecast = self.test_client.sessions.create_forecast('test-session-integration-forecast', 'observed',
                                                             dateutil.parser.parse('2008-09-01'),
                                                             dateutil.parser.parse('2008-09-30'))

        self.assertIsNotNone(forecast)
        self.assertEqual(SessionType.forecast, forecast.type)

    def test_estimate_forecast(self):
        self.test_client.datasets.create('test-session-integration-forecast', self.data)
        forecast = self.test_client.sessions.estimate_forecast('test-session-integration-forecast', 'observed',
                                                               dateutil.parser.parse('2008-09-01'),
                                                               dateutil.parser.parse('2008-09-30'))

        self.assertIsNotNone(forecast)
        self.assertTrue(forecast.is_estimate)
        self.assertEqual('0.01 USD', forecast.cost)

    def test_create_impact(self):
        self.test_client.datasets.create('test-session-integration-impact', self.data)
        impact = self.test_client.sessions.analyze_impact('test-session-integration-impact', 'observed',
                                                          'create-impact-test',
                                                          dateutil.parser.parse('2008-08-01'),
                                                          dateutil.parser.parse('2008-08-31'))

        self.assertIsNotNone(impact)

        self.assertEqual(SessionType.impact, impact.type)

    def test_estimate_impact(self):
        self.test_client.datasets.create('test-session-integration-impact', self.data)
        impact = self.test_client.sessions.estimate_impact('test-session-integration-impact', 'observed',
                                                           'create-impact-test',
                                                           dateutil.parser.parse('2008-08-01'),
                                                           dateutil.parser.parse('2008-08-31'))

        self.assertIsNotNone(impact)

        self.assertTrue(impact.is_estimate)
        self.assertEqual('0.01 USD', impact.cost)

    def test_get_results(self):
        results = self.test_client.sessions.get_results(self.forecast.session_id)

        self.assertEqual(len(results.data), 29)

    def test_remove_session(self):
        session = self.test_client.sessions.estimate_forecast(self.ds_name, 'observed',
                                                              dateutil.parser.parse('2008-09-01'),
                                                              dateutil.parser.parse('2008-09-30'))

        self.test_client.sessions.remove(session.session_id)

        try:
            self.test_client.sessions.get(session.session_id)
        except ClientError as e:
            self.assertEqual(404, e.status)
        else:
            self.assertFalse(True, 'Should have thrown a 404 error.')

    def test_create_session_no_data_error(self):
        try:
            self.test_client.sessions.create_forecast('does-not-exist', 'something',
                                                      dateutil.parser.parse('2008-09-01'),
                                                      dateutil.parser.parse('2008-09-30'))
        except ClientError as e:
            if e.status != 404:
                self.assertFalse(True, 'Should have thrown 404 error.')

    def test_create_session_bad_dates(self):
        try:
            self.test_client.sessions.create_forecast(self.ds_name, 'observed',
                                                      dateutil.parser.parse('2017-09-01'),
                                                      dateutil.parser.parse('2008-09-30'))
        except ClientError as e:
            if e.status != 400:
                self.assertFalse(True, 'Should have thrown 400 error.')

    def _setup_sessions(self):
        # check if we have a session data for forecast and impact, and if not, kick them off
        current_sessions = self.test_client.sessions.list(self.ds_name)
        for s in current_sessions:
            if s.type == SessionType.forecast:
                self.forecast = s
            if s.type == SessionType.impact:
                self.impact = s

        if self.forecast is None:
            self.forecast = self.test_client.sessions.create_forecast(self.ds_name, 'observed',
                                                                      dateutil.parser.parse('2008-09-01'),
                                                                      dateutil.parser.parse('2008-09-30'))
        if self.impact is None:
            self.impact = self.test_client.sessions.analyze_impact(self.ds_name, 'observed',
                                                                   'integration-test-analysis',
                                                                   dateutil.parser.parse('2008-08-01'),
                                                                   dateutil.parser.parse('2008-08-31'))

        counter = 0
        while self.impact.status != Status.completed and self.forecast.status != Status.completed:
            self.impact = self.test_client.sessions.get(self.impact.session_id)
            self.forecast = self.test_client.sessions.get(self.forecast.session_id)

            if counter > 120:
                raise TimeoutError('Running the sessions took longer than 10 minutes of setup time...')

            counter = counter + 1
            time.sleep(5)
